// XML Tools
// Jeffrey J Weimer
// SVN History: $Revision: 553 $ on $Date: 2008-02-27 15:48:27 -0500 (Wed, 27 Feb 2008) $

// Accessible Functions

// XMLPrint(property,text,[attributes, values, pretty])
// prints XML string to history
//	property: string of propert(ies)	- "property1:property2: ... propertyN"
//	text: string text to put at final property
//	attributes: string list of attributes		- "attribute1;attribute2; ... attributeN"
//	values: list of values for each attribute
//	pretty: variable 	- 0 (default) returns without tabs and returns, 1 returns with tabs and returns

// XMLPrintf(property,text,[attributes, values, pretty])
//	same as above but returns string rather than prints to history

// XMLWaveNote(twave,property,text,[attributes, values, pretty])
//	same as above but attaches text to wave twave (full path or current folder only!)

// XMLNoteText(ntbk,property,text,[attributes, values, pretty])
//	same as above but puts text to notebook ntbk

// VARIATIONS
// text or values prefaced with % are interpreted as STRING commands to be executed
// to override this, use \% (escape code sequence)

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version = 0.10

Function XMLPrint(property,text,[attributes, values, pretty])
	string property, text, attributes, values
	variable pretty

	string theText
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		theText = xmlprintf(property,text, pretty=pretty)
	else
		if (ParamIsDefault(values))
			theText = xmlprintf(property, text, attributes=attributes, values="", pretty=pretty)
		else
			theText = xmlprintf(property, text, attributes=attributes, values=values, pretty=pretty)
		endif
	endif
	print theText
	return 0
end

Function/S XMLPrintF(property,text,[attributes, values, pretty])
	string property, text, attributes, values
	variable pretty
	
	string theText
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		theText = parseXMLinput(property,text, pretty=pretty)
	else
		if (ParamIsDefault(values))
			theText = parseXMLinput(property, text, attributes=attributes, values="", pretty=pretty)
		else
			theText = parseXMLinput(property, text, attributes=attributes, values=values, pretty=pretty)
		endif
	endif
	return theText
end

Function XMLWaveNote(twave,property,text,[attributes, values, pretty])
	wave twave
	string property, text, attributes, values
	variable pretty

	string theText
	
	if (!WaveExists(twave))
		return -1
	endif
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		theText = xmlprintf(property,text, pretty=pretty)
	else
		if (ParamIsDefault(values))
			theText = xmlprintf(property, text, attributes=attributes, values="", pretty=pretty)
		else
			theText = xmlprintf(property, text, attributes=attributes, values=values, pretty=pretty)
		endif
	endif
	note twave, theText
	return 0
end	

Function XMLNoteText(ntbk,property,text,[attributes, values, pretty])
	string ntbk, property, text, attributes, values
	variable pretty

	string theText
	
	if (strlen(WinList(ntbk,";","WIN:16"))==0)
		return -1
	endif
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		theText = xmlprintf(property,text, pretty=pretty)
	else
		if (ParamIsDefault(values))
			theText = xmlprintf(property, text, attributes=attributes, values="", pretty=pretty)
		else
			theText = xmlprintf(property, text, attributes=attributes, values=values, pretty=pretty)
		endif
	endif
	theText += "\r"
	notebook $ntbk text=theText
	return 0
end	

Static Function/S parseXMLinput(property,text,[attributes, values, pretty])
	string property, text, attributes, values
	variable pretty

	string tS0 = "<", ts1 = "</", tE = ">", tR = "\r"
	string tT = "\t", tS = " ", tQ = "\""
	
	string theProp ="", rtnStr = "", theEnd = "", theAtt, itsV
	
	variable ic, nt, nv
	
	if (pretty!=1)
		tR = ""
		tT = ""
	endif
	
	if (ParamIsDefault(attributes))
		theAtt = ""
	else
		nt = ItemsInList(attributes)
		if (nt==1)
			itsV = parseXMLvalue(StringFromList(0,values))
			theAtt = tS + attributes + "=" + tQ + itsV + tQ
		else
			nv = ItemsInList(values)
			if (nv < nt)
				values = ""
				for(ic=0;ic<nt;ic+=1)
					values += ";"
				endfor
			endif
			theAtt = ""
			for(ic=0;ic<nt;ic+=1)
				theProp = StringFromList(ic,attributes)
				itsV = parseXMLvalue(StringFromList(ic,values))
				theAtt += tS + theProp + "=" + tQ + itsV + tQ
			endfor
		endif
	endif
	
	nt = ItemsInList(property,":")
	
	if (nt==1)
		theProp = StringFromList(0,property,":")
		rtnStr = ts0 + theProp + theAtt + tE + parseXMLvalue(text)
		rtnStr += ts1 + theProp + tE
	else
		for (ic=0;ic<nt-1;ic+=1)
			theProp = StringFromList(ic,property,":")
			theEnd += tT
			rtnStr += ts0 + theProp + tE + tR + theEnd
		endfor
		theProp = StringFromList(nt-1,property,":")
		rtnStr += ts0 + theProp + theAtt + tE + parseXMLvalue(text)
		rtnStr += ts1 + theProp + tE
		for (ic=nt-2;ic>=0;ic-=1)
			theEnd = RemoveEnding(theEnd,"\t")
			theProp = StringFromList(ic,property,":")
			rtnStr += tR + theEnd + ts1 + theProp + tE
		endfor
	endif
	
	return rtnStr
end

Static Function/S parseXMLvalue(vStr)
	string vStr
	
	string rtnStr, theCmd
	
	theCmd = vStr[0,1]
	if (cmpstr(theCmd,"\%")==0)
		rtnStr = vStr[1,strlen(vStr)-1]
		return rtnStr
	endif
	
	if (cmpstr(vStr[0],"%") == 0)
		string/G tmpXMLTools
		theCmd =  "tmpXMLTools = " + vStr[1,strlen(vStr)-1]
		Execute/Q theCmd
		rtnStr = tmpXMLTools
		killstrings/Z tmpXMLTools
	else
		rtnStr = vStr
	endif
	
	return rtnStr
end